# Copyright 2015,  A10 Networks
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

try:
    import neutron_lib.exceptions as lib_exceptions
except ImportError:
    lib_exceptions = None

try:
    import neutron.common.exceptions as old_exceptions
except ImportError:
    old_exceptions = None


def _find(*args):
    for f in args:
        try:
            return f()
        except AttributeError:
            pass


NotFound = _find(lambda: old_exceptions.NotFound,
                 lambda: lib_exceptions.NotFound)
InUse = _find(lambda: old_exceptions.InUse,
              lambda: lib_exceptions.InUse)
Conflict = _find(lambda: old_exceptions.Conflict,
                 lambda: lib_exceptions.Conflict)
