#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


LB_METHOD_ROUND_ROBIN = 'ROUND_ROBIN'
PROTOCOL_TERMINATED_HTTPS = 'TERMINATED_HTTPS'
PROTOCOL_HTTP = 'HTTP'
PROTOCOL_TCP = 'TCP'
PROTOCOL_HTTPS = 'HTTPS'

# Statuses for a10 objects (certificate bindings)
STATUS_CREATING = 0
STATUS_CREATED = 1
STATUS_DELETING = 2
